/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version. You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.aitools.programd.listener;

import java.util.HashMap;
import java.util.Map;

import com.levelonelabs.aim.AIMClient;
import com.levelonelabs.aim.AIMSender;

import org.aitools.programd.Core;
import org.aitools.programd.bot.Bot;
import org.aitools.programd.listener.Listener;
import org.apache.log4j.Logger;

/**
 * @author <a href="mailto:noel@aitools.org">Noel Bush</a>
 *
 */
public class AIMListener implements Listener
{
    /** The Core object in use. */
    protected Core core;

    /** The bot for which this listener works. */
    protected Bot bot;

    /** The id of the bot for which this listener works. */
    protected String botID;

    /** The parameters that can be set for this listener. */
    protected Map<String, String> parameters;

    /** The logger to use. */
    protected Logger logger = Logger.getLogger("programd.listener.irc");

    /** The userid. */
    private String userid;
    
    /** The password. */
    private String password;
    
    /** The AIM connection provider. */
    private AIMSender provider;

    /**
     * Constructs a new <code>ICQandAIMListener</code> listener and sets up
     * parameters.
     * 
     * @param coreToUse the Core object in use
     * @param botToListenFor the bot for whom to listen
     * @param parametersToUse the parameters for the listener and their default
     *            values
     */
    public AIMListener(Core coreToUse, Bot botToListenFor, HashMap<String, String> parametersToUse)
    {
        this.core = coreToUse;
        this.bot = botToListenFor;
        this.botID = botToListenFor.getID();
        this.parameters = parametersToUse;
        
        // Set parameters.
        this.userid = this.parameters.get("userid");
        this.password = this.parameters.get("password");
    }

    /**
     * @see org.aitools.programd.listener.Listener#checkParameters()
     */
    public void checkParameters() throws InvalidListenerParameterException
    {
        if (this.userid == null)
        {
            throw new InvalidListenerParameterException("Must specify a userid!");
        }
        if (this.userid.length() == 0)
        {
            throw new InvalidListenerParameterException("Userid cannot be zero-length!");
        }
        if (this.password == null)
        {
            throw new InvalidListenerParameterException("Must specify a password!");
        }
        if (this.password.length() == 0)
        {
            throw new InvalidListenerParameterException("Password cannot be zero-length!");
        }
    }

    /**
     * @see org.aitools.programd.util.ManagedProcess#shutdown()
     */
    public void shutdown()
    {
        // Log out.
        this.provider.signOff();
        this.logger.info("AIM listener logged out.");
    }

    /**
     * @see java.lang.Runnable#run()
     */
    public void run()
    {
        // Try to log in.
        this.logger.info("AIM listener logging in.");
        this.provider = new AIMClient(this.userid, this.password);
        this.provider.addAIMListener(new ProgramDAIMAdapter(this.provider, this.core, this.botID));
        this.provider.signOn();
    }
}
