/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version. You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.aitools.programd.listener;

import org.aitools.programd.Core;
import org.aitools.programd.util.XMLKit;
import org.apache.log4j.Logger;

import com.levelonelabs.aim.AIMAdapter;
import com.levelonelabs.aim.AIMBuddy;
import com.levelonelabs.aim.AIMSender;

/**
 * Implements the AIMAdapter for Program D.
 * 
 * @author <a href="mailto:noel@aitools.org">Noel Bush</a>
 *
 */
public class ProgramDAIMAdapter extends AIMAdapter
{
    private AIMSender provider;
    
    private Core core;
    
    private String botID;
    
    private Logger logger = Logger.getLogger("programd.listener.irc");
    
    /**
     * Instantiates one of these with an AIM provider, core and a bot id.
     * 
     * @param providerToUse
     * @param coreToUse
     * @param botIDToUse
     */
    public ProgramDAIMAdapter(AIMSender providerToUse, Core coreToUse, String botIDToUse)
    {
        this.provider = providerToUse;
        this.core = coreToUse;
        this.botID = botIDToUse;
    }
    
    /**
     * Gets a message and responds to it.
     * 
     * @param sender
     * @param request
     */
    @Override
    public void handleMessage(AIMBuddy sender, String request)
    {
        // Be sure the Core is ready.
        while (this.core.getStatus() != Core.Status.READY)
        {
            this.logger.warn("AIM listener: Waiting for Core to become ready.");
            try
            {
                Thread.sleep(1000);
            }
            catch (InterruptedException e)
            {
                // Nothing to do.
            }
        }
        String[] response = XMLKit.filterViaHTMLTags(this.core.getResponse(request, sender.getName(), this.botID));
        if (response.length > 0)
        {
            for (int line = 0; line < response.length; line++)
            {
                this.provider.sendMessage(sender, XMLKit.filterWhitespace(response[line]));
            }
        }
    }

}
